#ifndef CSG_TYPES_H_
#define CSG_TYPES_H_

#include <memory>
#include <optional>
#include <tuple>
#include <variant>
#include <vector>

namespace csg {

struct Sphere {
  double radius;
};

struct Cube {
  std::tuple<double, double, double> size;
  bool center;
};

struct Cylinder {
  double radius;
  double height;
  bool center;
};

struct Cone {
  double radius1;
  double radius2;
  double height;
  bool center;
};

struct Mulmatrix;
struct Union;
struct Intersection;
struct Difference;

using Type = std::variant<Sphere, Cube, Cylinder, Cone, Union, Intersection,
                          Difference, Mulmatrix>;

struct Union {
  std::vector<Type> objs;
};

struct Intersection {
  std::vector<Type> objs;
};

struct Difference {
  std::shared_ptr<Type> first_obj;
  Union next_objs;
};

struct Mulmatrix {
  std::array<std::array<double, 3>, 3> rotation;
  std::array<double, 3> translation;
  Union group;
};

struct Tree {
  Union top;
};

double signed_distance(const Union &, double, double, double);

} // namespace csg

#endif
